@@ -85,6 +85,7 @@ struct ipmi_session {
 	int password;
 	int port;
 	int active;
+	int retry;
 
 	uint32_t session_id;
 	uint32_t in_seq;
@@ -148,15 +149,28 @@ struct ipmi_session {
 	} sol_data;
 };
 
+struct ipmi_cmd {
+	int (*func)(struct ipmi_intf * intf, int argc, char ** argv);
+	const char * name;
+	const char * desc;
+};
+
+struct ipmi_intf_support {
+	const char * name;
+	int supported;
+};
+
 struct ipmi_intf {
 	char name[16];
 	char desc[128];
 	int fd;
 	int opened;
 	int abort;
+	int noanswer;
 
 	struct ipmi_session * session;
 	struct ipmi_oem_handle * oem;
+	struct ipmi_cmd * cmdlist;
 	uint32_t my_addr;
 	uint32_t target_addr;
 
@@ -171,7 +185,7 @@ struct ipmi_intf {
 };
 
 struct ipmi_intf * ipmi_intf_load(char * name);
-void ipmi_intf_print(void);
+void ipmi_intf_print(struct ipmi_intf_support * intflist);
 
 void ipmi_intf_session_set_hostname(struct ipmi_intf * intf, char * hostname);
 void ipmi_intf_session_set_username(struct ipmi_intf * intf, char * username);
@@ -180,6 +194,8 @@ void ipmi_intf_session_set_privlvl(struct ipmi_intf * intf, uint8_t privlvl);
 void ipmi_intf_session_set_cipher_suite_id(struct ipmi_intf * intf, uint8_t cipher_suite_id);
 void ipmi_intf_session_set_port(struct ipmi_intf * intf, int port);
 void ipmi_intf_session_set_authtype(struct ipmi_intf * intf, uint8_t authtype);
+void ipmi_intf_session_set_timeout(struct ipmi_intf * intf, uint32_t timeout);
+void ipmi_intf_session_set_retry(struct ipmi_intf * intf, int retry);
 void ipmi_cleanup(struct ipmi_intf * intf);
 
 #endif /* IPMI_INTF_H */
