@@ -108,6 +108,14 @@ get_lan_param(struct ipmi_intf * intf, uint8_t chan, int param)
 		p->data_len = 0;
 		return p;
 	}
+	if (rsp->ccode == 0xc9) {
+		/* parameter out of range
+		 * return lan_param without data
+		 */
+		p->data = NULL;
+		p->data_len = 0;
+		return p;
+	}
 	if (rsp->ccode > 0) {
 		lprintf(LOG_INFO, "Get LAN Parameter command failed: %s",
 			val2str(rsp->ccode, completion_code_vals));
@@ -256,6 +264,7 @@ __set_lan_param(struct ipmi_intf * intf, uint8_t chan,
  *  IPMI_LANP_WRITE_UNLOCK
  *  IPMI_LANP_WRITE_LOCK
  *  IPMI_LANP_WRITE_COMMIT
+ *  -1 on error/if not supported
  *
  * @intf:    ipmi interface handle
  * @chan:    ipmi channel
@@ -265,6 +274,8 @@ ipmi_lanp_lock_state(struct ipmi_intf * intf, uint8_t chan)
 {
 	struct lan_param * p;
 	p = get_lan_param(intf, chan, IPMI_LANP_SET_IN_PROGRESS);
+	if (p == NULL)
+		return -1;
 	return (p->data[0] & 3);
 }
 
@@ -285,7 +296,12 @@ ipmi_lanp_lock(struct ipmi_intf * intf, uint8_t chan)
 	uint8_t val = IPMI_LANP_WRITE_LOCK;
 	int retry = 3;
 
-	while (ipmi_lanp_lock_state(intf, chan) != val) {
+	for (;;) {
+		int state = ipmi_lanp_lock_state(intf, chan);
+		if (state == -1)
+			break;
+		if (state == val)
+			break;
 		if (retry-- == 0)
 			break;
 		__set_lan_param(intf, chan, IPMI_LANP_SET_IN_PROGRESS,
@@ -642,36 +658,33 @@ ipmi_lan_print(struct ipmi_intf * intf, uint8_t chan)
 	p = get_lan_param(intf, chan, IPMI_LANP_RMCP_CIPHER_SUPPORT);
 	if (p == NULL)
 		return -1;
-	else
+	else if (p->data != NULL)
 	{
-		unsigned char cipher_suite_count = p->data[0];
-
 		p = get_lan_param(intf, chan, IPMI_LANP_RMCP_CIPHERS);
 		if (p == NULL)
 			return -1;
 
 		printf("%-24s: ", p->desc);
-		if (cipher_suite_count == 0)
-			printf("None\n");
+
+		/* Now we're dangerous.  There are only 15 fixed cipher
+		   suite IDs, but the spec allows for 16 in the return data.*/
+		if ((p->data != NULL) && (p->data_len <= 16))
+		{
+			unsigned char cipher_suite_count = p->data[0];
+			unsigned int i;
+			for (i = 0; (i < 16) && (i < cipher_suite_count); ++i)
+			{
+				printf("%s%d",
+				       (i > 0? ",": ""),
+				       p->data[i + 1]);
+			}
+			printf("\n");
+		}
 		else
 		{
-			/* Now we're dangerous.  There are only 15 fixed cipher
-			   suite IDs, but the spec allows for 16 in the return data.*/
-			if ((p->data != NULL) && (p->data_len <= 16))
-			{
-				unsigned int i;
-				for (i = 0; (i < 16) && (i < cipher_suite_count); ++i)
-				{
-					printf("%s%d",
-					       (i > 0? ",": ""),
-					       p->data[i + 1]);
-				}
-				printf("\n");
-			}
+			printf("None\n");
 		}
 	}
-	
-
 
 	
 	/* RMCP+ Messaging Cipher Suite Privilege Levels */
