@@ -223,7 +223,7 @@ struct ipmi_rs * ipmi_lan_recv_packet(struct ipmi_intf * intf)
 		return NULL;
 	}
 
-	alarm(IPMI_LAN_TIMEOUT);
+	alarm(intf->session->timeout);
 	rc = recv(intf->fd, &rsp.data, IPMI_BUF_SIZE, 0);
 	alarm(0);
 
@@ -238,7 +238,7 @@ struct ipmi_rs * ipmi_lan_recv_packet(struct ipmi_intf * intf)
 	 * response is read before the connection refused is returned)
 	 */
 	if (rc < 0) {
-		alarm(IPMI_LAN_TIMEOUT);
+		alarm(intf->session->timeout);
 		rc = recv(intf->fd, &rsp.data, IPMI_BUF_SIZE, 0);
 		alarm(0);
 		if (rc < 0) {
@@ -691,7 +691,7 @@ struct ipmi_rs *
 ipmi_lan_send_cmd(struct ipmi_intf * intf, struct ipmi_rq * req)
 {
 	struct ipmi_rq_entry * entry;
-	struct ipmi_rs * rsp;
+	struct ipmi_rs * rsp = NULL;
 	int try = 0;
 
 	if (intf->opened == 0 && intf->open != NULL) {
@@ -712,6 +712,10 @@ ipmi_lan_send_cmd(struct ipmi_intf * intf, struct ipmi_rq * req)
 			continue;
 		}
 
+		/* if we are set to noanswer we do not expect response */
+		if (intf->noanswer)
+			break;
+
 		if (ipmi_oem_active(intf, "intelwv2"))
 			ipmi_lan_thump(intf);
 
@@ -722,7 +726,7 @@ ipmi_lan_send_cmd(struct ipmi_intf * intf, struct ipmi_rq * req)
 			break;
 
 		usleep(5000);
-		if (++try >= IPMI_LAN_RETRY) {
+		if (++try >= intf->session->retry) {
 			lprintf(LOG_DEBUG, "  No response from remote controller");
 			break;
 		}
@@ -1181,7 +1185,8 @@ ipmi_set_session_privlvl_cmd(struct ipmi_intf * intf)
 
 	if (rsp->ccode > 0) {
 		lprintf(LOG_ERR, "Set Session Privilege Level to %s failed: %s",
-			val2str(privlvl, ipmi_privlvl_vals));
+			val2str(privlvl, ipmi_privlvl_vals),
+			val2str(rsp->ccode, completion_code_vals));
 		return -1;
 	}
 
@@ -1285,12 +1290,12 @@ ipmi_lan_activate_session(struct ipmi_intf * intf)
 	if (rc < 0)
 		goto fail;
 
+	intf->abort = 0;
+
 	rc = ipmi_set_session_privlvl_cmd(intf);
 	if (rc < 0)
 		goto fail;
 
-	intf->abort = 0;
-
 	return 0;
 
  fail:
@@ -1331,6 +1336,10 @@ int ipmi_lan_open(struct ipmi_intf * intf)
 		s->port = IPMI_LAN_PORT;
 	if (s->privlvl == 0)
 		s->privlvl = IPMI_SESSION_PRIV_ADMIN;
+	if (s->timeout == 0)
+		s->timeout = IPMI_LAN_TIMEOUT;
+	if (s->retry == 0)
+		s->retry = IPMI_LAN_RETRY;
 
 	if (s->hostname == NULL || strlen(s->hostname) == 0) {
 		lprintf(LOG_ERR, "No hostname specified!");
