@@ -285,6 +285,35 @@ static const struct valstr plus_payload_types_vals[] = {
 	{ 0x00, NULL },
 };
 
+
+static struct ipmi_rq_entry *
+ipmi_req_add_entry(struct ipmi_intf * intf, struct ipmi_rq * req)
+{
+	struct ipmi_rq_entry * e;
+
+	e = malloc(sizeof(struct ipmi_rq_entry));
+	if (e == NULL) {
+		lprintf(LOG_ERR, "ipmitool: malloc failure");
+		return NULL;
+	}
+
+	memset(e, 0, sizeof(struct ipmi_rq_entry));
+	memcpy(&e->req, req, sizeof(struct ipmi_rq));
+
+	e->intf = intf;
+
+	if (ipmi_req_entries == NULL)
+		ipmi_req_entries = e;
+	else
+		ipmi_req_entries_tail->next = e;
+
+	ipmi_req_entries_tail = e;
+	lprintf(LOG_DEBUG+3, "added list entry seq=0x%02x cmd=0x%02x",
+		e->rq_seq, e->req.msg.cmd);
+	return e;
+}
+
+
 static struct ipmi_rq_entry *
 ipmi_req_lookup_entry(uint8_t seq, uint8_t cmd)
 {
@@ -309,8 +338,8 @@ ipmi_req_remove_entry(uint8_t seq, uint8_t cmd)
 		e = e->next;
 	}
 	if (e) {
-		if (verbose > 3)
-			printf("removed list entry seq=0x%02x cmd=0x%02x\n", seq, cmd);
+		lprintf(LOG_DEBUG+3, "removed list entry seq=0x%02x cmd=0x%02x",
+			seq, cmd);
 		p->next = (p->next == e->next) ? NULL : e->next;
 		if (ipmi_req_entries == e) {
 			if (ipmi_req_entries != p)
@@ -324,6 +353,7 @@ ipmi_req_remove_entry(uint8_t seq, uint8_t cmd)
 			else
 				ipmi_req_entries_tail = NULL;
 		}
+		
 		if (e->msg_data)
 			free(e->msg_data);
 		free(e);
@@ -337,9 +367,8 @@ ipmi_req_clear_entries(void)
 
 	e = ipmi_req_entries;
 	while (e) {
-		if (verbose > 3)
-			printf("cleared list entry seq=0x%02x cmd=0x%02x\n",
-			       e->rq_seq, e->req.msg.cmd);
+		lprintf(LOG_DEBUG+3, "cleared list entry seq=0x%02x cmd=0x%02x",
+			e->rq_seq, e->req.msg.cmd);
 		p = e->next;
 		free(e);
 		e = p;
@@ -1399,7 +1428,7 @@ ipmi_lanplus_build_v2x_msg(
 		sizeof(rmcp)                +  // RMCP Header (4)
 		10                          +  // IPMI Session Header
 		2                           +  // Message length
-		IPMI_MAX_PAYLOAD_SIZE       +  // The actual payload
+		payload->payload_length     +  // The actual payload
 		IPMI_MAX_INTEGRITY_PAD_SIZE +  // Integrity Pad
 		1                           +  // Pad Length
 		1                           +  // Next Header
@@ -1515,6 +1544,7 @@ ipmi_lanplus_build_v2x_msg(
 	default:
 		lprintf(LOG_ERR, "unsupported payload type 0x%x",
 			payload->payload_type);
+		free(msg);
 		assert(0);
 		break;
 	}
@@ -1565,13 +1595,20 @@ ipmi_lanplus_build_v2x_msg(
 		 * Determine the required integrity pad length.  We have to make the
 		 * data range covered by the authcode a multiple of 4.
 		 */
-		uint32_t length_before_authcode =
-			12                          + /* the stuff before the payload */
-			payload->payload_length     +
-			1                           + /* pad length field  */
-			1;                            /* next header field */
+		uint32_t length_before_authcode;
+
+		if (ipmi_oem_active(intf, "icts")) {
+			length_before_authcode =
+				12                          + /* the stuff before the payload */
+				payload->payload_length;
+		} else {
+			length_before_authcode =
+				12                          + /* the stuff before the payload */
+				payload->payload_length     +
+				1                           + /* pad length field  */
+				1;                            /* next header field */
+		}
 
-		
 		if (length_before_authcode % 4)
 			integrity_pad_size = 4 - (length_before_authcode % 4);
 							  
@@ -1663,29 +1700,10 @@ ipmi_lanplus_build_v2x_ipmi_cmd(
  	static uint8_t curr_seq = 0;
 	if (curr_seq >= 64)
 		curr_seq = 0;
-	
-	entry = malloc(sizeof(struct ipmi_rq_entry));
-	if (entry == NULL) {
-		lprintf(LOG_ERR, "ipmitool: malloc failure");
+
+	entry = ipmi_req_add_entry(intf, req);
+	if (entry == NULL)
 		return NULL;
-	}
-	memset(entry, 0, sizeof(struct ipmi_rq_entry));
-	memcpy(&entry->req, req, sizeof(struct ipmi_rq));
-
-	entry->intf = intf;
-	entry->rq_seq = curr_seq;
-
-	// Factor out
-	if (!ipmi_req_entries)
-		ipmi_req_entries = entry;
-	else
-		ipmi_req_entries_tail->next = entry;
-	ipmi_req_entries_tail = entry;
-
-
-	lprintf(LOG_DEBUG+3, "added list entry seq=0x%02x cmd=0x%02x\n",
-		curr_seq, req->msg.cmd);
-
 
 	// Build our payload
 	v2_payload.payload_type                 = IPMI_PAYLOAD_TYPE_IPMI;
@@ -1748,23 +1766,9 @@ ipmi_lanplus_build_v15_ipmi_cmd(
 	struct ipmi_session  * session = intf->session;
 	struct ipmi_rq_entry * entry;
 
-	entry = malloc(sizeof(struct ipmi_rq_entry));
-	if (entry == NULL) {
-		lprintf(LOG_ERR, "ipmitool: malloc failure");
+	entry = ipmi_req_add_entry(intf, req);
+	if (entry == NULL)
 		return NULL;
-	}
-	memset(entry, 0, sizeof(struct ipmi_rq_entry));
-	memcpy(&entry->req, req, sizeof(struct ipmi_rq));
-
-	entry->intf = intf;
-
-	// Can be factored out
-	if (!ipmi_req_entries)
-		ipmi_req_entries = entry;
-	else
-		ipmi_req_entries_tail->next = entry;
-
-	ipmi_req_entries_tail = entry;
 
 	len = req->msg.data_len + 21;
 
@@ -2010,7 +2014,7 @@ ipmi_lanplus_send_payload(
 	}
 
 
-	while (try < IPMI_LAN_RETRY) {
+	while (try < session->retry) {
 
 
 		if (ipmi_lan_send_packet(intf, msg_data, msg_length) < 0) {
@@ -2018,6 +2022,10 @@ ipmi_lanplus_send_payload(
 			return NULL;
 		}
 
+		/* if we are set to noanswer we do not expect response */
+		if (intf->noanswer)
+			break;
+
 		usleep(100); 			/* Not sure what this is for */
 
 		/* Remember our connection state */
@@ -2094,8 +2102,13 @@ ipmi_lanplus_send_payload(
 
 	
 	/* IPMI messages are deleted under ipmi_lan_poll_recv() */
-	if (payload->payload_type == IPMI_PAYLOAD_TYPE_SOL)
+	switch (payload->payload_type) {
+	case IPMI_PAYLOAD_TYPE_RMCP_OPEN_REQUEST:
+	case IPMI_PAYLOAD_TYPE_RAKP_1:
+	case IPMI_PAYLOAD_TYPE_RAKP_3:
 		free(msg_data);
+		break;
+	}
 
 	return rsp;
 }
@@ -2528,7 +2541,10 @@ ipmi_lanplus_open_session(struct ipmi_intf * intf)
 	memset(msg, 0, IPMI_OPEN_SESSION_REQUEST_SIZE);
 
 	msg[0] = 0; /* Message tag */
-	msg[1] = 0; /* Give us highest privlg level based on supported algorithms */
+	if (ipmi_oem_active(intf, "intelplus") || session->privlvl != IPMI_SESSION_PRIV_ADMIN)
+		msg[1] = session->privlvl;
+	else
+		msg[1] = 0; /* Give us highest privlg level based on supported algorithms */
 	msg[2] = 0; /* reserved */
 	msg[3] = 0; /* reserved */
 
@@ -2547,6 +2563,7 @@ ipmi_lanplus_open_session(struct ipmi_intf * intf)
 	{
 		lprintf(LOG_WARNING, "Unsupported cipher suite ID : %d\n",
 				session->cipher_suite_id);
+		free(msg);
 		return -1;
 	}
 
@@ -2594,6 +2611,7 @@ ipmi_lanplus_open_session(struct ipmi_intf * intf)
 
 	rsp = ipmi_lanplus_send_payload(intf, &v2_payload);
 
+	free(msg);
 
 	if (verbose)
 		lanplus_dump_open_session_response(rsp);
@@ -2723,6 +2741,7 @@ ipmi_lanplus_rakp1(struct ipmi_intf * intf)
 		// ERROR;
 		lprintf(LOG_ERR, "ERROR generating random number "
 			"in ipmi_lanplus_rakp1");
+		free(msg);
 		return 1;
 	}
 	memcpy(msg + 8, session->v2_data.console_rand, 16);
@@ -2753,6 +2772,7 @@ ipmi_lanplus_rakp1(struct ipmi_intf * intf)
 		lprintf(LOG_ERR, "ERROR: user name too long.  "
 			"(Exceeds %d characters)",
 			IPMI_MAX_USER_NAME_LENGTH);
+		free(msg);
 		return 1;
 	}
 	memcpy(msg + 28, session->username, msg[27]);
@@ -2764,6 +2784,8 @@ ipmi_lanplus_rakp1(struct ipmi_intf * intf)
 
 	rsp = ipmi_lanplus_send_payload(intf, &v2_payload);
 
+	free(msg);
+
 	if (rsp == NULL)
 	{
 		lprintf(LOG_INFO, "> Error: no response from RAKP 1 message");
@@ -2790,7 +2812,8 @@ ipmi_lanplus_rakp1(struct ipmi_intf * intf)
 		memcpy(session->v2_data.bmc_rand, rsp->payload.rakp2_message.bmc_rand, 16);
 		memcpy(session->v2_data.bmc_guid, rsp->payload.rakp2_message.bmc_guid, 16);
 
-		printbuf(session->v2_data.bmc_rand, 16, "bmc_rand");
+		if (verbose > 2)
+			printbuf(session->v2_data.bmc_rand, 16, "bmc_rand");
 
 		/*
 		 * It is at this point that we have to decode the random number and determine
@@ -2883,6 +2906,7 @@ ipmi_lanplus_rakp3(struct ipmi_intf * intf)
 		{
 			/* Error */
 			lprintf(LOG_INFO, "> Error generating RAKP 3 authcode");
+			free(msg);
 			return 1;
 		}
 		else
@@ -2896,18 +2920,21 @@ ipmi_lanplus_rakp3(struct ipmi_intf * intf)
 		{
 			/* Error */
 			lprintf(LOG_INFO, "> Error generating session integrity key");
+			free(msg);
 			return 1;
 		}
 		else if (lanplus_generate_k1(session))
 		{
 			/* Error */
 			lprintf(LOG_INFO, "> Error generating K1 key");
+			free(msg);
 			return 1;
 		}
 		else if (lanplus_generate_k2(session))
 		{
 			/* Error */
 			lprintf(LOG_INFO, "> Error generating K1 key");
+			free(msg);
 			return 1;
 		}
 	}
@@ -2915,6 +2942,8 @@ ipmi_lanplus_rakp3(struct ipmi_intf * intf)
 
 	rsp = ipmi_lanplus_send_payload(intf, &v2_payload);
 
+	free(msg);
+
 	if (session->v2_data.rakp2_return_code != IPMI_RAKP_STATUS_NO_ERRORS)
 	{
 		/*
@@ -3014,7 +3043,11 @@ ipmi_lanplus_open(struct ipmi_intf * intf)
 	if (!session->port)
 		session->port = IPMI_LANPLUS_PORT;
 	if (!session->privlvl)
-		session->privlvl = IPMI_SESSION_PRIV_USER;
+		session->privlvl = IPMI_SESSION_PRIV_ADMIN;
+	if (!session->timeout)
+		session->timeout = IPMI_LAN_TIMEOUT;
+	if (!session->retry)
+		session->retry = IPMI_LAN_RETRY;
 
 	if (session->hostname == NULL || strlen(session->hostname) == 0) {
 		lprintf(LOG_ERR, "No hostname specified!");
@@ -3035,7 +3068,6 @@ ipmi_lanplus_open(struct ipmi_intf * intf)
 	//session->sol_data.last_received_byte_count      = 0;
 	memset(session->v2_data.sik, 0, IPMI_SIK_BUFFER_SIZE);
 	memset(session->v2_data.kg,  0, IPMI_KG_BUFFER_SIZE);
-	session->timeout = IPMI_LAN_TIMEOUT;
 
 
 	/* open port to BMC */
@@ -3152,7 +3184,7 @@ ipmi_lanplus_open(struct ipmi_intf * intf)
 
 
 
-void test_crypt1()
+void test_crypt1(void)
 {
 	uint8_t key[]  =
 		{0x01, 0x02, 0x03, 0x04, 0x05, 0x06, 0x07, 0x08, 0x09, 0x0A,
@@ -3201,7 +3233,7 @@ void test_crypt1()
 
 
 
-void test_crypt2()
+void test_crypt2(void)
 {
 	uint8_t key[]  =
 		{0x01, 0x02, 0x03, 0x04, 0x05, 0x06, 0x07, 0x08, 0x09, 0x0A,
